︠43b81320-909d-44e4-bb48-2455ddc6d55di︠
%html
<h1>ESBS</h1>
<h1>Biophysics practicals 2012</h1>
<p>This SAGE notebook contains some hints and programs that will allow you to analyse the data recorded during the experiments. Exemples are provided to show you the more appropriate way to analyse and display your data. The graphics could be exported directly to fit within your reports as png files.</p>
<h2>1. TP 2 Mass transport : Non-linear fitting of the time dependant evolution of the gel's temperature</h2>
<p>The time evolution of the gel's temperature could be modelled using the following equation:</p>
<p>$$T(t)=A+B\left(1-e^{-Ct}\right)$$</p>
<p>where $$A=T_{0},&nbsp;&nbsp;&nbsp; B=\frac{P}{aC}&nbsp;&nbsp; and &nbsp; C=a$$</p>
<ul>
<li><strong>First :</strong> Get an idea of the effect of each of the parameters on the curve using the following interactive plot</li>
</ul>
<p>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; In particular, check which parameter influences 1) the value of the plateau 2) the initial slope</p>
<p>&nbsp;</p>

︡c1db7638-161c-4381-9331-1f2ae5199219︡{"html": "<h1>ESBS</h1>\r\n<h1>Biophysics practicals 2012</h1>\r\n<p>This SAGE notebook contains some hints and programs that will allow you to analyse the data recorded during the experiments. Exemples are provided to show you the more appropriate way to analyse and display your data. The graphics could be exported directly to fit within your reports as png files.</p>\r\n<h2>1. TP 2 Mass transport : Non-linear fitting of the time dependant evolution of the gel's temperature</h2>\r\n<p>The time evolution of the gel's temperature could be modelled using the following equation:</p>\r\n<p>$$T(t)=A+B\\left(1-e^{-Ct}\\right)$$</p>\r\n<p>where $$A=T_{0},&nbsp;&nbsp;&nbsp; B=\\frac{P}{aC}&nbsp;&nbsp; and &nbsp; C=a$$</p>\r\n<ul>\r\n<li><strong>First :</strong> Get an idea of the effect of each of the parameters on the curve using the following interactive plot</li>\r\n</ul>\r\n<p>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; In particular, check which parameter influences 1) the value of the plateau 2) the initial slope</p>\r\n<p>&nbsp;</p>"}︡
︠430037ee-c324-4c25-8356-6616fc988a6e︠
@interact
def _(A=(10..100),B=(1..10),CI=(1..20)):
    C=CI*0.001
    f(t) = A + B*(1-exp(-C*t))
    pf = plot(f,0,1000,axes_labels=['Time (s)','Temp (Deg C)'])
    show(pf)
︡ccbac4a9-9375-4128-b1ce-6b4a8b9f159b︡︡
︠397d1d2f-a647-4512-8190-a9daec3ef3bdi︠
%html
<ul>
<li><strong>Second :</strong> We should define the function to be fitted, in a way that will be recognized by NUMPY</li>
</ul>

︡8754a166-7f07-4684-904d-21403f0af6da︡{"html": "<ul>\r\n<li><strong>Second :</strong> We should define the function to be fitted, in a way that will be recognized by NUMPY</li>\r\n</ul>"}︡
︠c342952a-296f-4621-a3fb-58a595b83a45︠
from numpy import *
def temp(time,pA,pB,pC):
    """ Equation describing heat exchange within an electrophoresis gel"""
    y= pA+pB*(1 - exp(-time*pC))
    return y
︡123b028d-4483-4c76-a80b-e643f0d6f291︡︡
︠c16c5c89-1a2e-48f0-abae-c91a43f56900i︠
%html
<ul>
<li><span id="cell_outer_5"><strong>Third :</strong> The measured values have to be entered in the program. First option : enter the values manually as shown below</span></li>
</ul>

︡5ed25262-a267-45de-8694-4692c4c99fdf︡{"html": "<ul>\r\n<li><span id=\"cell_outer_5\"><strong>Third :</strong> The measured values have to be entered in the program. First option : enter the values manually as shown below</span></li>\r\n</ul>"}︡
︠21edb50b-d98f-4b27-b93d-d0b1c9a11862︠
Times = [10, 30, 100, 300, 400, 600, 800, 1200, 2000]
Texp =  [20.2, 20.5, 21.3, 22.9, 23.3, 23.7, 23.9, 24.0, 24.0]
︡006f1de5-d322-4aa2-95b7-e5320324983b︡︡
︠fabff85b-bbfd-416f-8b6b-37f1d6df7d5ai︠
%html
<p>Second option: download a text file named as you like (just set the right value for the fname variable), that contains the recorded temperatures according the following format:</p>
<p>Fri Mar 16 17:31:34 2012 23.56<br />Fri Mar 16 17:31:40 2012 23.5</p>
<p>And run the following script</p>
<p>...</p>
<p>&nbsp;</p>

︡b7b35a13-3009-4367-befa-0f6bf4bc355b︡{"html": "<p>Second option: download a text file named as you like (just set the right value for the fname variable), that contains the recorded temperatures according the following format:</p>\r\n<p>Fri Mar 16 17:31:34 2012 23.56<br />Fri Mar 16 17:31:40 2012 23.5</p>\r\n<p>And run the following script</p>\r\n<p>...</p>\r\n<p>&nbsp;</p>"}︡
︠93e4898d-34ad-4f73-a994-712dfe66ade7︠
# arduino prep
# Read Arduino data 
# B. Kieffer Mar 2012

import time 

# 1. Read data into a list all_line
fname = 'arduino.dat'

myfile = open(DATA+fname,'r')
all_lines = myfile.readlines()
myfile.close()

txtime = []
Texp = []
Times = []
# 2. Extract relevent informations from all_line
for line in all_lines:
	line_field = line.split()
	time_field = line_field[3]
	val = line_field[5]
	txtime.append(time_field)
	Texp.append(float(val))

# 3. Build a vector of time delays starting from the first point
first = True	
for t in txtime :
   t1 = time.strptime(t, "%H:%M:%S")
   t_sec = t1.tm_hour*3600+t1.tm_min*60+t1.tm_sec
   if first :
      Times.append(0)
      t0 = t_sec # First point is the time 0
      first = False
   else :
      Times.append(t_sec-t0)
print " %i lines read from file %s " % (len(Times),fname)
︡84ea37d6-49d3-4663-bcdf-8de5c0179142︡{"stdout": "61 lines read from file arduino.dat"}︡
︠386dc095-3ecc-47b3-b53d-9c66c29ae235︠
Texp
︡5cc65529-4816-46e7-8f2f-f4eee4d4334e︡{"stdout": "[20.0, 20.190000000000001, 20.5, 20.75, 21.059999999999999, 21.25, 21.5, 21.75, 21.870000000000001, 22.059999999999999, 22.25, 22.370000000000001, 22.559999999999999, 22.690000000000001, 22.75, 22.870000000000001, 22.940000000000001, 23.059999999999999, 23.190000000000001, 23.370000000000001, 23.440000000000001, 23.5, 23.690000000000001, 23.690000000000001, 23.75, 23.870000000000001, 23.940000000000001, 24.0, 24.059999999999999, 24.120000000000001, 24.309999999999999, 24.309999999999999, 24.309999999999999, 24.370000000000001, 24.370000000000001, 24.440000000000001, 24.620000000000001, 24.620000000000001, 24.620000000000001, 24.75, 24.75, 24.75, 24.809999999999999, 24.940000000000001, 25.0, 25.059999999999999, 25.0, 24.940000000000001, 25.0, 25.059999999999999, 25.059999999999999, 25.190000000000001, 25.25, 25.190000000000001, 25.25, 25.190000000000001, 25.120000000000001, 25.190000000000001, 25.190000000000001, 25.120000000000001, 25.25]"}︡
︠623b2d42-c6a7-435d-8305-cfbc06889a73︠
Times
︡222cf8bd-7ddc-4753-af51-9735d0051877︡{"stdout": "[0, 31, 60, 91, 120, 150, 180, 210, 240, 270, 301, 330, 361, 390, 421, 450, 481, 510, 541, 570, 601, 630, 661, 690, 721, 750, 780, 810, 840, 870, 900, 931, 960, 991, 1020, 1051, 1080, 1111, 1140, 1171, 1200, 1231, 1260, 1291, 1320, 1351, 1380, 1410, 1440, 1470, 1500, 1530, 1560, 1590, 1621, 1650, 1681, 1710, 1741, 1770, 1801]"}︡
︠53d48107-86cd-4bc5-96c0-8b3aa423b01fi︠
%html
<ul>
<li><span id="cell_outer_5"><span id="cell_outer_5"><strong>Finally :</strong> Perform the non-linear fit using the program curve_fit that is included in the optimize library of SCIPY<br /></span></span></li>
</ul>

︡4a1027b6-a43f-4cb3-80be-59eb681bfed3︡{"html": "<ul>\r\n<li><span id=\"cell_outer_5\"><span id=\"cell_outer_5\"><strong>Finally :</strong> Perform the non-linear fit using the program curve_fit that is included in the optimize library of SCIPY<br /></span></span></li>\r\n</ul>"}︡
︠027383c8-e0c4-42ed-bf4f-9df1ae26c4fc︠
from scipy import optimize
# 1. Initialize parameters
Pini = [18., 2., 0.001] # Initial guess of the parameters
Pow = 10.0 # Power in the gel

# 2. Perform the fit
PP, pcov = optimize.curve_fit(temp, array(Times), array(Texp), Pini)

# 3. Report and plot the results
pA = PP[0] ; pB = PP[1] ; pC = PP[2]
Cap = Pow/(pB*pC)
dA = pcov[0][0]^0.5
dB = pcov[1][1]^0.5
dC = pcov[2][2]^0.5
dcpr = dB/pB + dC/pC
dCap = dcpr*Cap

print "Optimized parameters:"
print "T0: %f +/- %f (Deg C) " % (pA,dA)
print "Calorific capacity: %f +/- %f (J.K-1)" % (Cap,dCap)
print "Rate: %f +/- %f (s-1)" %(pC,dC)
a = scatter_plot(zip(Times,Texp))
a += plot(temp(x,PP[0],PP[1],PP[2]), 0,max(Times),axes_labels=['Time (s)','Temp (Deg C)'])
show(a)
︡9d5bfc0c-e964-4691-bd0f-90db79574f02︡{"stdout": "Optimized parameters:\nT0: 20.059797 +/- 0.032958 (Deg C) \nCalorific capacity: 1141.509034 +/- 28.951827 (J.K-1)\nRate: 0.001589 +/- 0.000030 (s-1)"}︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠27a20269-c44c-43b8-b21b-f56730627e33i︠
%html
<ul>
<li><span id="cell_outer_5"><span id="cell_outer_5"><strong>Note on error estimates :</strong> <br /></span></span></li>
</ul>
<p>The program <em>optimize.curve_fit</em> returns, along with the optimized parameters, a covariance matrix on these parameters. The diagonal terms on this matrix provides you with the variance on the parameters. This variance is a good estimate of the uncertainties, provided that your model is correct and that the noise is random.</p>
<p>The standard deviation on the T0 (initial temperature) and a (heating rate) can be calculated directly from the variance of the first and last parameter (A and C)</p>
<p><span style="font-family: courier new,courier;">dA = pcov[0][0]^0.5<br />dC = pcov[2][2]^0.5</span></p>
<p>The error on the Heat capacity value, however, should be obtained from a error propagation calculation, since its value results from two parameters B and C :</p>
<p>$$\frac{\Delta{Cap}}{Cap} = \frac{\Delta{B}}{B}+\frac{\Delta{C}}{C}$$</p>
<p>with $$Cap = \frac{Pow}{BxC}$$</p>
<p>B and C beeing the optimized parameters.</p>

<ul>
<li><strong>Assess the nature of the noise</strong></li>
</ul>
<p>It is useful to analyse the distribution of the experimental points around those calculated from the model. In particular, the shape of the distribution will help to assess the random or systematic nature of the noise.</p>
<p>&nbsp;</p>

︡217811a4-e354-4c87-93e0-fd865de070c4︡{"html": "<ul>\r\n<li><span id=\"cell_outer_5\"><span id=\"cell_outer_5\"><strong>Note on error estimates :</strong> <br /></span></span></li>\r\n</ul>\r\n<p>The program <em>optimize.curve_fit</em> returns, along with the optimized parameters, a covariance matrix on these parameters. The diagonal terms on this matrix provides you with the variance on the parameters. This variance is a good estimate of the uncertainties, provided that your model is correct and that the noise is random.</p>\r\n<p>The standard deviation on the T0 (initial temperature) and a (heating rate) can be calculated directly from the variance of the first and last parameter (A and C)</p>\r\n<p><span style=\"font-family: courier new,courier;\">dA = pcov[0][0]^0.5<br />dC = pcov[2][2]^0.5</span></p>\r\n<p>The error on the Heat capacity value, however, should be obtained from a error propagation calculation, since its value results from two parameters B and C :</p>\r\n<p>$$\\frac{\\Delta{Cap}}{Cap} = \\frac{\\Delta{B}}{B}+\\frac{\\Delta{C}}{C}$$</p>\r\n<p>with $$Cap = \\frac{Pow}{BxC}$$</p>\r\n<p>B and C beeing the optimized parameters.</p>\r\n\r\n<ul>\r\n<li><strong>Assess the nature of the noise</strong></li>\r\n</ul>\r\n<p>It is useful to analyse the distribution of the experimental points around those calculated from the model. In particular, the shape of the distribution will help to assess the random or systematic nature of the noise.</p>\r\n<p>&nbsp;</p>"}︡
︠c573d22c-b862-45a2-8a35-54b8690c8409︠
import numpy as np
YE = Texp
YC = []
for ts in Times :
    YC.append(temp(ts,PP[0],PP[1],PP[2]))    
YE = array(YE)
YC = array(YC)
YD = list(YE-YC)
a,b = np.histogram(YD, bins=20)
print b
bar_chart(a)
︡062ef104-486a-44cf-b92d-025428852e7d︡{"stdout": "[-0.13478857 -0.1211206  -0.10745263 -0.09378466 -0.08011669 -0.06644872\n -0.05278075 -0.03911278 -0.02544481 -0.01177684  0.00189113  0.01555911\n  0.02922708  0.04289505  0.05656302  0.07023099  0.08389896  0.09756693\n  0.1112349   0.12490287  0.13857084]"}︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
